<?php

/*
 * Plugin Name: Flat Cost Estimation Form
 * Version: 1.6
 * Plugin URI: http://www.20script.ir
 * Description: This plugin allows you to create easily a flat visual form of cost estimation.
 * Author: Biscay Charly (loopus)
 * Author URI: http://www.20script.ir
 * Requires at least: 3.8
 * Tested up to: 3.8.1
 *
 * @package WordPress
 * @author Biscay Charly (loopus)
 * @since 1.0.0
 */

if (!defined('ABSPATH'))
    exit;

register_activation_hook(__FILE__, 'wpecf_install');
register_deactivation_hook(__FILE__, 'wpecf_uninstall');

global $jal_db_version;
$jal_db_version = "1.0";

// Include plugin class files
require_once ('includes/wpe-tools.php');
require_once ('includes/wpe-admin-menu.php');
require_once ('includes/steps-items/Steps_List_Table.php');
require_once ('includes/steps-items/Items_List_Table.php');

function Estimation_Form() {
    $instance = WPE_Tools::instance(__FILE__, '1.1.0');
    if (is_null($instance->menu)) {
        $instance->menu = WPE_admin_menu::instance($instance);
    }

    return $instance;
}


/**
 * Return settings.
 * @access  public
 * @since   1.0.0
 * @return  void
 */
function getSettings() {
    global $wpdb;
    $table_name = $wpdb->prefix . "wpefc_settings";
    $settings = $wpdb->get_results("SELECT * FROM $table_name WHERE id=1 LIMIT 1");
    return $settings[0];
}

// End getSettings()

/**
 * Installation. Runs on activation.
 * @access  public
 * @since   1.0.0
 * @return  void
 */
function wpecf_install() {
    global $wpdb;
    global $jal_db_version;
    require_once( ABSPATH . '/wp-admin/includes/upgrade.php' );

    // create steps table
    $db_table_name = $wpdb->prefix . "wpefc_steps";
    if ($wpdb->get_var("SHOW TABLES LIKE '$db_table_name'") != $db_table_name) {
        if (!empty($wpdb->charset))
            $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
        if (!empty($wpdb->collate))
            $charset_collate .= " COLLATE $wpdb->collate";

        $sql = "CREATE TABLE $db_table_name (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		title VARCHAR(120) NOT NULL,
		ordersort mediumint(9) NOT NULL,
		itemRequired BOOL,
		UNIQUE KEY id (id)
		) $charset_collate;";
        dbDelta($sql);
    }
    // create items table
    $db_table_name = $wpdb->prefix . "wpefc_items";
    if ($wpdb->get_var("SHOW TABLES LIKE '$db_table_name'") != $db_table_name) {
        if (!empty($wpdb->charset))
            $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
        if (!empty($wpdb->collate))
            $charset_collate .= " COLLATE $wpdb->collate";

        $sql = "CREATE TABLE $db_table_name (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		title VARCHAR(120) NOT NULL,
		ordersort mediumint(9) NOT NULL,
		image VARCHAR(250) NOT NULL,
		groupitems VARCHAR(120) NOT NULL,
		type VARCHAR(120) NOT NULL,
		stepID mediumint(9) NOT NULL,
		price FLOAT NOT NULL,
		ischecked BOOL,
                isRequired BOOL,
		quantity_enabled BOOL,
		quantity_max SMALLINT(5)  NOT NULL,
		isWooLinked BOOL,
		wooProductID SMALLINT(5)  NOT NULL,
                imageTint BOOL,
		UNIQUE KEY id (id)
		) $charset_collate;";
        dbDelta($sql);
    }
    
    // create fields table
    $db_table_name = $wpdb->prefix . "wpefc_fields";
    if ($wpdb->get_var("SHOW TABLES LIKE '$db_table_name'") != $db_table_name) {
        if (!empty($wpdb->charset))
            $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
        if (!empty($wpdb->collate))
            $charset_collate .= " COLLATE $wpdb->collate";

        $sql = "CREATE TABLE $db_table_name (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		label VARCHAR(120) NOT NULL,
		ordersort mediumint(9) NOT NULL,
		isRequired BOOL,
		typefield VARCHAR(32) NOT NULL,
		visibility VARCHAR(32) NOT NULL,
		UNIQUE KEY id (id)
		) $charset_collate;";
        dbDelta($sql);
    }
    $wpdb->insert($db_table_name, array('label' => "Enter your email", 'isRequired' => 1,'typefield'=>'input','visibility'=>'display'));
    $wpdb->insert($db_table_name, array('label' => "Do you want to write a message ?", 'isRequired' => 0,'typefield'=>'textarea','visibility'=>'toggle'));


    // create settings table
    $db_table_name = $wpdb->prefix . "wpefc_settings";
    if ($wpdb->get_var("SHOW TABLES LIKE '$db_table_name'") != $db_table_name) {
        if (!empty($wpdb->charset))
            $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
        if (!empty($wpdb->collate))
            $charset_collate .= " COLLATE $wpdb->collate";

        $sql = "CREATE TABLE $db_table_name (
                id mediumint(9) NOT NULL AUTO_INCREMENT,
                btn_step VARCHAR(120) NOT NULL,
                previous_step VARCHAR(120) NOT NULL,
                intro_title VARCHAR(120) NOT NULL,
                intro_text TEXT NOT NULL,
                intro_btn VARCHAR(120) NOT NULL,
                last_title VARCHAR(120) NOT NULL,
                last_text TEXT NOT NULL,
                last_btn VARCHAR(120) NOT NULL,
                last_msg_label VARCHAR(240) NOT NULL,
                initial_price FLOAT NOT NULL,
                succeed_text TEXT NOT NULL,
                email VARCHAR(250) NOT NULL,
                email_subject VARCHAR(250) NOT NULL,
                currency VARCHAR (32) NOT NULL,
                errorMessage VARCHAR(240) NOT NULL,
                colorA VARCHAR(16) NOT NULL,
                colorB VARCHAR(16) NOT NULL,
                intro_enabled BOOL,
                save_to_cart BOOL,
                form_page_id SMALLINT(9) NOT NULL,
                item_pictures_size SMALLINT(9) NOT NULL,
                close_url VARCHAR(250) NOT NULL DEFAULT '#',
                gravityFormID SMALLINT(9) NOT NULL,
                UNIQUE KEY id (id)
		) $charset_collate;";
        dbDelta($sql);
    }

// Create form page 
    $_p = array();
    $_p['post_title'] = "Cost Estimation Form";
    $_p['post_content'] = "";
    $_p['post_status'] = 'publish';
    $_p['post_type'] = 'page';
    $_p['comment_status'] = 'closed';
    $_p['ping_status'] = 'closed';
    $_p['post_category'] = array(1);
    $page_id = wp_insert_post($_p);
    update_post_meta($page_id, '_wp_page_template', 'estimation-popup-page.php');

    // default settings
    $table_name = $wpdb->prefix . "wpefc_settings";
    $rows_affected = $wpdb->insert($table_name, array('btn_step' => "NEXT STEP", 'previous_step' => "return to previous step", 'intro_title' => "HOW MUCH TO MAKE MY WEBSITE ?", 'intro_text' => "Estimate the cost of a website easily using this awesome tool.", 'intro_btn' => "GET STARTED", 'last_title' => "Final cost", 'last_text' => "The final estimated price is : ", 'last_btn' => "ORDER MY WEBSITE", 'last_msg_label' => "Do you want to write a message ? ", 'succeed_text' => "Thanks, we will contact you soon", 'initial_price' => 0, 'email' => 'loopus_web@hotmail.fr', 'email_subject' => 'New order from your website', 'currency' => '$', 'errorMessage' => 'You need to select an item to continue', 'colorA' => '#1abc9c', 'colorB' => '#34495e', 'intro_enabled' => 1, 'form_page_id' => $page_id, 'item_pictures_size' => 64));

    add_option("jal_db_version", $jal_db_version);

    global $isInstalled;
    $isInstalled = true;
}

// End install()

/**
 * Uninstallation.
 * @access  public
 * @since   1.0.0
 * @return  void
 */
function wpecf_uninstall() {
    global $wpdb;
    global $jal_db_version;

    $settings = getSettings();

    $table_name = $wpdb->prefix . "wpefc_steps";
    $wpdb->query("DROP TABLE IF EXISTS $table_name");
    $table_name = $wpdb->prefix . "wpefc_items";
    $wpdb->query("DROP TABLE IF EXISTS $table_name");
    $table_name = $wpdb->prefix . "wpefc_settings";
    $wpdb->query("DROP TABLE IF EXISTS $table_name");
    $table_name = $wpdb->prefix . "wpefc_fields";
    $wpdb->query("DROP TABLE IF EXISTS $table_name");


    wp_delete_post($settings->form_page_id, true);
}

// End uninstall()


Estimation_Form();
