<?php
if (!defined('ABSPATH'))
    exit;

class WPE_admin_menu {

    /**
     * The single instance 
     * @var 	object
     * @access  private
     * @since 	1.0.0
     */
    private static $_instance = null;

    /**
     * The main plugin object.
     * @var 	object
     * @access  public
     * @since 	1.0.0
     */
    public $parent = null;

    /**
     * Prefix for plugin settings.
     * @var     string
     * @access  publicexport
     * 
     * @since   1.0.0
     */
    public $base = '';

    /**
     * Available settings for plugin.
     * @var     array
     * @access  public
     * @since   1.0.0
     */
    public $settings = array();

    /**
     * Is WooCommerce activated ?
     * @var     array
     * @access  public
     * @since   1.5.0
     */
    public $isWooEnabled = false;

    public function __construct($parent) {
        $this->parent = $parent;
        $this->base = 'wpt_';
        $this->dir = dirname($this->parent->file);

        add_action('admin_menu', array($this, 'add_menu_item'));
        add_action('admin_print_scripts', array($this, 'admin_scripts'));
        add_action('admin_print_styles', array($this, 'admin_styles'));
        add_action('wp_ajax_nopriv_step_save', array($this, 'step_save'));
        add_action('wp_ajax_step_save', array($this, 'step_save'));
        add_action('wp_ajax_nopriv_item_save', array($this, 'item_save'));
        add_action('wp_ajax_item_save', array($this, 'item_save'));
        add_action('wp_ajax_nopriv_settings_save', array($this, 'settings_save'));
        add_action('wp_ajax_settings_save', array($this, 'settings_save'));
        add_action('wp_ajax_nopriv_field_save', array($this, 'field_save'));
        add_action('wp_ajax_field_save', array($this, 'field_save'));
        add_filter('plugin_action_links_' . plugin_basename($this->parent->file), array($this, 'add_settings_link'));



        $this->check_form_page();
    }

    /**
     * Add settings link to plugin list table
     * @param  array $links Existing links
     * @return array 		Modified links
     */
    public function add_settings_link($links) {
        $settings_link = '<a href="admin.php?page=wpefc_menu">' . __('Settings', 'wpe') . '</a>';
        array_push($links, $settings_link);
        return $links;
    }

    /**
     * Add menu to admin
     * @return void
     */
    public function add_menu_item() {
        add_menu_page('Estimation Form', 'Estimation Form', 'manage_options', 'wpefc_menu', array($this, 'estimation_submenu_settings'), 'dashicons-feedback');
        add_submenu_page('wpefc_menu', 'Steps', 'Steps', 'manage_options', 'wpe-steps', array($this, 'estimation_submenu_steps'));
        add_submenu_page('wpefc_menu', 'Edit Step', 'Add new Step', 'manage_options', 'wpe-step-add', array($this, 'estimation_submenu_step_add'));
        add_submenu_page('wpefc_menu', 'Items', 'Items', 'manage_options', 'wpe-items', array($this, 'estimation_submenu_items'));
        add_submenu_page('wpefc_menu', 'Edit Item', 'Add new Item', 'manage_options', 'wpe-item-add', array($this, 'estimation_submenu_item_add'));
        add_submenu_page('wpefc_menu', 'Import', 'Import', 'manage_options', 'wpe-import', array($this, 'estimation_submenu_import'));
        add_submenu_page('wpefc_menu', 'Export', 'Export', 'manage_options', 'wpe-export', array($this, 'estimation_submenu_export'));
        add_submenu_page('wpefc_menu', 'Edit Field', 'Add field', 'manage_options', 'wpe-field-edit', array($this, 'estimation_render_field_add'));
    }

    /*
     *  check if a page with the form template exists
     * 
     */

    private function check_form_page() {
        global $wpdb;
        global $isInstalled;
        if ($isInstalled) {
            require_once( ABSPATH . '/wp-admin/includes/upgrade.php' );
            $settings = $this->getSettings();
            $page = get_post($settings->form_page_id);
            $template_file = get_post_meta($settings->form_page_id, '_wp_page_template', TRUE);
            if ($settings->form_page_id == 0 || !$template_file || $template_file != "estimation-popup-page.php") {
                $args = array('post_type' => 'page', 'posts_per_page' => 99);
                $pages = get_posts($args);
                $selPageID = 0;
                foreach ($pages as $page) {
                    if (get_post_meta($page->ID, '_wp_page_template', TRUE) == "estimation-popup-page.php") {
                        $selPageID = $page->ID;
                    }
                }
                if ($selPageID > 0) {
                    $table_name = $wpdb->prefix . "wpefc_settings";
                    $wpdb->update($table_name, array('form_page_id' => $selPageID), array('id' => 1));
                } else {
                    
                }
            }
        }
    }

    /**
     * Return settings.
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    public function getSettings() {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_settings";
        $settings = $wpdb->get_results("SELECT * FROM $table_name WHERE id=1 LIMIT 1");
        if ($settings[0]) {
            return $settings[0];
        } else {
            return false;
        }
    }

// End getSettings()

    /**
     * remove specific item
     * @return void
     */
    private function remove_item($item_id) {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_items";
        $wpdb->delete($table_name, array('id' => $item_id));
    }

    /**
     * Menu steps items render
     * @return void
     */
    function estimation_submenu_items() {
        if (isset($_GET['remove'])) {
            $this->remove_item($_GET['remove']);
        }

        $itemsTable = new Items_List_Table();
        $itemsTable->prepare_items();
        ?>
        <div class="wrap">
            <div id="icon-users" class="icon32"></div>
            <h2> Items list <a href="admin.php?page=wpe-item-add" class="add-new-h2">Add New</a></h2>

            <?php $itemsTable->display(); ?>
        </div>
        <?php
    }

    /**
     * remove specific field
     * @return void
     */
    private function remove_field($field_id) {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_fields";
        $wpdb->delete($table_name, array('id' => $field_id));
    }

    /**
     * remove specific step
     * @return void
     */
    private function remove_step($step_id) {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_steps";
        $wpdb->delete($table_name, array('id' => $step_id));
    }

    /**
     * Menu steps render
     * @return void
     */
    function estimation_submenu_steps() {
        if (isset($_GET['remove'])) {
            $this->remove_step($_GET['remove']);
        }

        $stepTable = new Steps_List_Table();
        $stepTable->prepare_items();
        ?>
        <div class="wrap">
            <div id="icon-users" class="icon32"></div>
            <h2> Steps list <a href="admin.php?page=wpe-step-add" class="add-new-h2">Add New</a></h2>

            <?php $stepTable->display(); ?>
        </div>
        <?php
    }

    /**
     * Load admin scripts
     * @return void
     */
    function admin_scripts() {
        if (isset($_GET['page']) && $_GET['page'] == 'wpe-item-add') {
            wp_enqueue_script('jquery');
            wp_enqueue_script('media-upload');
            wp_enqueue_script('thickbox');
        }
    }

    function admin_styles() {
        if (isset($_GET['page']) && $_GET['page'] == 'wpe-item-add') {
            wp_enqueue_style('thickbox');
        }
    }

    /**
     * Get specific field datas
     * @return object
     */
    private function getFieldDatas($field_id) {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_fields";
        $rows = $wpdb->get_results("SELECT * FROM $table_name WHERE id=$field_id LIMIT 1");
        return $rows[0];
    }

    /**
     * Get  fields datas
     * @return object
     */
    private function getFieldsDatas() {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_fields";
        $rows = $wpdb->get_results("SELECT * FROM $table_name  ORDER BY ordersort ASC");
        return $rows;
    }

    /**
     * Get specific Item datas
     * @return object
     */
    private function getItemDatas($item_id) {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_items";
        $rows = $wpdb->get_results("SELECT * FROM $table_name WHERE id=$item_id LIMIT 1");
        return $rows[0];
    }

    /**
     * Get specific Step datas
     * @return object
     */
    private function getStepDatas($step_id) {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_steps";
        $rows = $wpdb->get_results("SELECT * FROM $table_name WHERE id=$step_id LIMIT 1");
        return $rows[0];
    }

    /**
     * Get Steps datas
     * @return Array
     */
    private function getStepsData() {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_steps";
        $rows = $wpdb->get_results("SELECT * FROM $table_name");

        $data = array();
        foreach ($rows as $row) {
            $data[] = array('id' => $row->id, 'title' => $row->title, 'order' => $row->ordersort);
        }
        return $data;
    }

    /**
     * Get steps groups
     * @return Array
     */
    private function getGroups() {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_items";
        $rows = $wpdb->get_results("SELECT * FROM $table_name");

        $groups = array();
        foreach ($rows as $row) {
            $groups[] = $row->groupitems;
        }
        return $groups;
    }

    /**
     * Menu add item render
     * @return void
     */
    function estimation_render_field_add() {
        $datas = false;
        if (isset($_GET['field'])) {
            $datas = $this->getFieldDatas($_GET['field']);
        }
        ?>
        <div class="wrap">
            <h2>Edit a field</h2>
            <div id="wpefc_response"></div>
            <form method="post" action="#" onsubmit="qc_process(this);
                            return false;">
                <input id="id" type="hidden" name="id" value="<?php
                if ($datas) {
                    echo $datas->id;
                } else {
                    echo '0';
                }
                ?>"/>
                <table class="form-table">
                    <tbody>
                        <tr>
                            <th scope="row">Label</th>
                            <td>
                                <input id="label" type="text" name="label" placeholder="Field label" value="<?php
                                if ($datas) {
                                    echo $datas->label;
                                }
                                ?>">
                                <label for="label"> <span class="description">This is the field label</span> </label></td>
                        </tr>
                        <tr>
                            <th scope="row">Order</th>
                            <td>
                                <input id="ordersort" type="number" name="ordersort" placeholder="Item order" value="<?php
                                if ($datas) {
                                    echo $datas->ordersort;
                                } else {
                                    echo '0';
                                }
                                ?>">
                                <label for="sortorder">
                                    <span class="description">Fields take place according to the order defined</span>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Is required ?</th>
                            <td>
                                <select id="isRequired" name="isRequired" placeholder="Is required ?">
                                    <?php
                                    $sel1 = '';
                                    $sel2 = '';
                                    if ($datas && $datas->isRequired) {
                                        $sel2 = 'selected';
                                    } else {
                                        $sel1 = 'selected';
                                    }
                                    echo '<option value="0" ' . $sel1 . '>No</option>';
                                    echo '<option value="1" ' . $sel2 . '>Yes</option>';
                                    ?>
                                </select>
                                <label for="isRequired">
                                    <span class="description">Is the field required to validate the form ?</span>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Type of field</th>
                            <td>
                                <select id="typefield" name="typefield" placeholder="Field type">
                                    <?php
                                    $sel1 = '';
                                    $sel2 = '';
                                    if ($datas && $datas->typefield == 'textarea') {
                                        $sel2 = 'selected';
                                    } else {
                                        $sel1 = 'selected';
                                    }
                                    echo '<option value="input" ' . $sel1 . '>Input</option>';
                                    echo '<option value="textarea" ' . $sel2 . '>Textarea</option>';
                                    ?>
                                </select>
                                <label for="typefield">
                                    <span class="description">Is the field required to validate the form ?</span>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Toogle or display field ?</th>
                            <td>
                                <select id="visibility" name="visibility" placeholder="Display">
                                    <?php
                                    $sel1 = '';
                                    $sel2 = '';
                                    if ($datas && $datas->visibility == 'toggle') {
                                        $sel2 = 'selected';
                                    } else {
                                        $sel1 = 'selected';
                                    }
                                    echo '<option value="display" ' . $sel1 . '>Displayed</option>';
                                    echo '<option value="toggle" ' . $sel2 . '>Toggle</option>';
                                    ?>
                                </select>
                                <label for="visibility">
                                    <span class="description">Is the field required to validate the form ?</span>
                                </label>
                            </td>
                        </tr>

                        <tr>
                            <th scope="row"></th>
                            <td>
                                <input type="submit" value="Save" class="button-primary"/>
                            </td>                            
                        </tr>
                    </tbody>
                </table>
                <script>
                    function qc_process(e) {

                        var error = false;
                        jQuery('#label').removeClass('field-error');
                        if (jQuery("#label").val().length < 3) {
                            error = true;
                            jQuery('#label').addClass('field-error');
                        }

                        if (!error) {
                            var data = {
                                action: "field_save",
                                id: e["id"].value,
                                label: e["label"].value,
                                ordersort: e["ordersort"].value,
                                typefield: e["typefield"].value,
                                isRequired: e["isRequired"].value,
                                visibility: e["visibility"].value
                            };
                            jQuery("#wpefc_response").hide();
                            jQuery.post(ajaxurl, data, function(response) {
                                jQuery("#wpefc_response").html(response);
                                jQuery("#wpefc_response").fadeIn();
                                setTimeout(function() {
                                    document.location.href = "admin.php?page=wpefc_menu&tab=last";
                                }, 400);
                            });
                        }

                    }
        <?php
        if ($datas && $datas->id == 1) {
            ?>
                        jQuery(document).ready(function() {
                            jQuery('#typefield').parent().parent().hide();
                            jQuery('#visibility').parent().parent().hide();
                            jQuery('#isRequired').parent().parent().hide();
                        });
            <?php
        }
        ?>
                </script>
            </form>
        </div>
        <?php
    }

    /**
     * Menu add item render
     * @return void
     */
    function estimation_submenu_item_add() {
        wp_enqueue_style('thickbox');
        wp_enqueue_script('thickbox');
        $stepsData = $this->getStepsData();
        $datas = false;
        if (isset($_GET['item'])) {
            $datas = $this->getItemDatas($_GET['item']);
        }
        //$groups = $this->getGroups();
        ?>
        <div class="wrap">
            <h2>Edit an item</h2>
            <div id="wpefc_response"></div>
            <form method="post" action="#" onsubmit="qc_process(this);
                            return false;">
                <input id="id" type="hidden" name="id" value="<?php
                if ($datas) {
                    echo $datas->id;
                } else {
                    echo '0';
                }
                ?>"/>
                <table class="form-table">
                    <tbody>

                        <?php
                        if (is_plugin_active('woocommerce/woocommerce.php')) {
                            $disp = '';
                        } else {
                            $disp = 'style="display:none;"';
                        }
                        ?>
                        <tr <?php echo $disp; ?>>
                            <th scope="row">Woocommerce product</th>
                            <td>
                                <select id="wooProductID" name="wooProductID">
                                    <?php
                                    echo '<option value="0">None</option>';
                                    if (is_plugin_active('woocommerce/woocommerce.php')) {
                                        $args = array('post_type' => 'product', 'posts_per_page' => 99, 'orderby' => 'category', 'order' => 'ASC');
                                        $products = get_posts($args);
                                        foreach ($products as $productI) {
                                            $product = new WC_Product($productI->ID);
                                            $cat = '';
                                            $cats = $product->get_categories(',');
                                            $cats = explode(',', $cats);
                                            foreach ($cats as $catI) {
                                                $cat = $cat . $catI . ' > ';
                                            }
                                            $sel = '';
                                            $dataMax = '';
                                            $dataImg = '';
                                            if ($datas && $datas->wooProductID == $productI->ID) {
                                                $sel = 'selected';
                                            }
                                            if ($product->get_stock_quantity() && $product->get_stock_quantity() > 0) {
                                                if ($product->get_stock_quantity() > 5) {
                                                    $dataMax = 'data-max="5"';
                                                } else {
                                                    $dataMax = 'data-max="' . $product->get_stock_quantity() . '"';
                                                }
                                            }
                                            // check image
                                            $argsI = array('post_type' => 'attachment', 'numberposts' => -1, 'post_status' => null, 'post_parent' => $productI->ID);
                                            $attachments = get_posts($argsI);
                                            if ($attachments[0]) {
                                                $imgDom = wp_get_attachment_image($attachments[count($attachments) - 1]->ID, 'thumbnail');
                                                $img = substr($imgDom, strpos($imgDom, 'src="') + 5, strpos($imgDom, '"', stripos($imgDom, 'src="') + 6) - (strpos($imgDom, 'src="') + 5));

                                                $dataImg = 'data-img="' . $img . '"';
                                            }

                                            echo '<option ' . $sel . ' ' . $dataImg . ' ' . $dataMax . ' value="' . $productI->ID . '" data-title="' . $productI->post_title . '">' . $cat . $productI->post_title . '</option>';
                                        }
                                    }
                                    ?>
                                </select>
                                <label for="wooProductID">
                                    <span class="description">You can select a product from your catalog</span>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Title</th>
                            <td>
                                <input id="title" type="text" name="title" placeholder="Item name" value="<?php
                                if ($datas) {
                                    echo $datas->title;
                                }
                                ?>">
                                <label for="title"> <span class="description">This is the item name</span> </label></td>
                        </tr>
                        <tr>
                            <th scope="row">Step</th>
                            <td>
                                <select id="stepID" name="stepID" placeholder="Select step">
                                    <?php
                                    foreach ($stepsData as $step) {
                                        $sel = '';
                                        if ($datas && $step['id'] == $datas->stepID) {
                                            $sel = 'selected';
                                        }
                                        echo '<option value="' . $step['id'] . '" ' . $sel . '>' . $step['title'] . '</value>';
                                    }
                                    ?>
                                </select>
                                <label for="order">
                                    <span class="description">Enter the step of the item</span>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Type</th>
                            <td>
                                <select id="type" name="type" placeholder="Select type">
                                    <?php
                                    $sel1 = '';
                                    $sel2 = '';
                                    $sel3 = '';
                                    if ($datas && $datas->type == 'checkbox') {
                                        $sel2 = 'selected';
                                    } else if ($datas && $datas->type == 'textfield') {
                                        $sel3 = 'selected';
                                    } else {
                                        $sel1 = 'selected';
                                    }
                                    echo '<option value="picture" ' . $sel1 . '>Picture</value>';
                                    echo '<option value="checkbox" ' . $sel2 . '>Checkbox</value>';
                                    echo '<option value="textfield" ' . $sel3 . '>Textfield</value>';
                                    ?>
                                </select>
                                <label for="type">
                                    <span class="description">Image, checkbox or textfield ?</span>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Order</th>
                            <td>
                                <input id="order" type="number" name="order" placeholder="Item order" value="<?php
                                if ($datas) {
                                    echo $datas->ordersort;
                                } else {
                                    echo '0';
                                }
                                ?>">
                                <label for="order">
                                    <span class="description">Items take place according to the order defined</span>
                                </label>
                            </td>
                        </tr>
                        <tr class="picOnly">
                            <th scope="row">Image</th>
                            <td>
                                <input id="image" type="text" name="image" value="<?php
                                if ($datas) {
                                    echo $datas->image;
                                }
                                ?>" size="50" />
                                <input  class="imageBtn button" type="button" value="Upload Image" />
                                <label for="image">
                                    <span class="description">Choose file if the item is a picture</span>
                                </label></td>
                        </tr>                        
                        <tr class="picOnly">
                            <th scope="row">Tint image ?</th>
                            <td>
                                <select id="imageTint" name="imageTint" placeholder="Tint image ?">
                                    <?php
                                    $sel1 = '';
                                    $sel2 = '';
                                    if ($datas && $datas->imageTint) {
                                        $sel2 = 'selected';
                                    } else {
                                        $sel1 = 'selected';
                                    }
                                    echo '<option value="0" ' . $sel1 . '>No</option>';
                                    echo '<option value="1" ' . $sel2 . '>Yes</option>';
                                    ?>
                                </select>
                                <label for="imageTint"> <span class="description">Automaticly fill the picture with the main color</span> </label></td>
                        </tr>
                        <tr class="picOnly">
                            <th scope="row">Group</th>
                            <td>
                                <input id="group" type="text" name="group" placeholder="Optional group name" value="<?php
                                if ($datas) {
                                    echo $datas->groupitems;
                                }
                                ?>">
                                <label for="group"> <span class="description">If you enter a group name, a single item of the group will be clickable</span> </label></td>
                        </tr>

                        <tr class="picOnly">
                            <th scope="row">Enable quantity choice ?</th>
                            <td>
                                <select id="quantity_enabled" name="quantity_enabled" placeholder="Is quantity enabled ?">
                                    <?php
                                    $sel1 = '';
                                    $sel2 = '';
                                    if ($datas && $datas->quantity_enabled) {
                                        $sel2 = 'selected';
                                    } else {
                                        $sel1 = 'selected';
                                    }
                                    echo '<option value="0" ' . $sel1 . '>No</option>';
                                    echo '<option value="1" ' . $sel2 . '>Yes</option>';
                                    ?>
                                </select>
                                <label for="quantity_enabled"> <span class="description">Can the user select a quantity for this item ?</span> </label></td>
                        </tr>

                        <tr class="picOnly" id="quantity_max_tr">
                            <th scope="row">Max quantity</th>
                            <td>
                                <input id="quantity_max" type="number" name="quantity_max" placeholder="Max quantity" value="<?php
                                if ($datas) {
                                    echo $datas->quantity_max;
                                } else {
                                    echo '3';
                                }
                                ?>">
                                <label for="quantity_max"> <span class="description">Sets the maximum quanity that can be selected</span> </label></td>
                        </tr>
                        <tr class="wooMasked">
                            <th scope="row">Price</th>
                            <td>
                                <input id="price" type="number" name="price" placeholder="Item price" value="<?php
                                if ($datas) {
                                    echo $datas->price;
                                }
                                ?>">
                                <label for="price"> <span class="description">Sets the item price</span> </label></td>
                        </tr>
                        <tr>
                            <th scope="row">Is selected ?</th>
                            <td>
                                <select id="ischecked" name="ischecked" placeholder="Is checked ?">
                                    <?php
                                    $sel1 = '';
                                    $sel2 = '';
                                    if ($datas && $datas->ischecked) {
                                        $sel2 = 'selected';
                                    } else {
                                        $sel1 = 'selected';
                                    }
                                    echo '<option value="0" ' . $sel1 . '>No</option>';
                                    echo '<option value="1" ' . $sel2 . '>Yes</option>';
                                    ?>
                                </select>
                                <label for="ischecked">
                                    <span class="description">Is the item selected by default ?</span>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Is required ?</th>
                            <td>
                                <select id="isRequired" name="isRequired" placeholder="Is required ?">
                                    <?php
                                    $sel1 = '';
                                    $sel2 = '';
                                    if ($datas && $datas->isRequired) {
                                        $sel2 = 'selected';
                                    } else {
                                        $sel1 = 'selected';
                                    }
                                    echo '<option value="0" ' . $sel1 . '>No</option>';
                                    echo '<option value="1" ' . $sel2 . '>Yes</option>';
                                    ?>
                                </select>
                                <label for="isRequired">
                                    <span class="description">Is the item required to continue ?</span>
                                </label>
                            </td>
                        </tr>



                        <tr>
                            <th scope="row"></th>
                            <td>
                                <input type="submit" value="Save" class="button-primary"/>
                            </td>
                        </tr>
                    </tbody>
                </table>
            </form>
        </div>
        <script>
            function qc_process(e) {

                var error = false;
                jQuery('#title').removeClass('field-error');
                jQuery('#image').removeClass('field-error');
                jQuery('#quantity_max').removeClass('field-error');

                if (jQuery("#title").val().length < 3) {
                    error = true;
                    jQuery('#title').addClass('field-error');
                }
                if (jQuery('#type').val() == 'picture' && jQuery("#image").val().length < 4) {
                    error = true;
                    jQuery('#image').addClass('field-error');
                }
                if (jQuery('#quantity_enabled').val() == '1' && jQuery("#quantity_max").val() == "") {
                    error = true;
                    jQuery('#quantity_max').addClass('field-error');
                }

                if (!error) {
                    var data = {
                        action: "item_save",
                        id: e["id"].value,
                        title: e["title"].value,
                        order: e["order"].value,
                        image: e["image"].value,
                        stepID: e["stepID"].value,
                        group: e["group"].value,
                        price: e["price"].value,
                        type: e["type"].value,
                        ischecked: e["ischecked"].value,
                        quantity_enabled: e["quantity_enabled"].value,
                        quantity_max: e["quantity_max"].value,
                        wooProductID: e["wooProductID"].value,
                        imageTint: e["imageTint"].value,
                        isRequired: e["isRequired"].value
                    };
                    jQuery("#wpefc_response").hide();
                    jQuery.post(ajaxurl, data, function(response) {
                        jQuery("#wpefc_response").html(response);
                        jQuery("#wpefc_response").fadeIn();
                    });
                }

            }

            function itemTypeUpdated() {
                if (jQuery('#type').val() == 'picture') {
                    jQuery('.picOnly').show();
                    jQuery('#price').parent().parent().show();
                    jQuery('#isRequired').parent().parent().hide();
                } else {
                    jQuery('#isRequired').parent().parent().show();
                    jQuery('.picOnly').hide();
                    if (jQuery('#type').val() == 'textfield') {
                        jQuery('#price').parent().parent().hide();
                    } else {
                        jQuery('#price').parent().parent().show();
                    }
                }
            }

            function quantityEnabledUpdated() {
                if (jQuery('#quantity_enabled').val() == '1') {
                    jQuery('#quantity_max_tr').show();
                } else {
                    jQuery('#quantity_max_tr').hide();
                }
            }

            function quantityUpdated() {
                if (jQuery('#quantityUpdated').val() < 1) {
                    jQuery('#quantityUpdated').val('3');
                }
            }

            var isWoo = false;
            function wooUpdated() {
                if (jQuery('#wooProductID').val() != '0') {
                    if (!isWoo) {
                        jQuery('#quantity_enabled').val('1');
                        jQuery('#quantity_max_tr').show();
                    }
                    isWoo = true;
                    jQuery('.wooMasked').fadeOut(250);
                    jQuery("#title").val(jQuery('#wooProductID option:selected').data('title'));
                    if (jQuery('#wooProductID option:selected').data('max')) {
                        jQuery("#quantity_max").val(jQuery('#wooProductID option:selected').data('max'));
                    }
                    if (jQuery('#wooProductID option:selected').data('img')) {
                        jQuery("#image").val(jQuery('#wooProductID option:selected').data('img'));
                    }
                } else {
                    isWoo = false;
                    jQuery('.wooMasked').fadeIn(250);
                }

            }


            jQuery(document).ready(function() {

                jQuery('#type').change(itemTypeUpdated);
                itemTypeUpdated();

                jQuery('#quantity_enabled').change(quantityEnabledUpdated);
                quantityEnabledUpdated();

                jQuery('#quantity_max').change(quantityUpdated);
                quantityUpdated();

                jQuery('#wooProductID').change(wooUpdated);
                if (jQuery('#wooProductID').val() != '0') {
                    isWoo = true;
                    jQuery('.wooMasked').fadeOut(250);
                }

                var formfield;
                jQuery('.imageBtn').click(function() {
                    formfield = jQuery(this).prev('input');
                    tb_show('', 'media-upload.php?TB_iframe=true');

                    return false;

                });
                window.old_tb_remove = window.tb_remove;
                window.tb_remove = function() {
                    window.old_tb_remove();
                    formfield = null;
                };
                window.original_send_to_editor = window.send_to_editor;
                window.send_to_editor = function(html) {
                    if (formfield) {
                        fileurl = jQuery('img', html).attr('src');
                        jQuery(formfield).val(fileurl);
                        tb_remove();
                    } else {
                        window.original_send_to_editor(html);
                    }
                };

            });
        </script>
        <?php
    }

    /**
     * Menu add step render
     * @return void
     */
    function estimation_submenu_step_add() {
        $datas = false;
        if (isset($_GET['step'])) {
            $datas = $this->getStepDatas($_GET['step']);
        }
        ?>
        <div class="wrap">
            <h2>Edit a step</h2>
            <div id="wpefc_response"></div>
            <form method="post" action="#" onsubmit="qc_process(this);
                            return false;">
                <input id="id" type="hidden" name="id" value="<?php
                if ($datas) {
                    echo $datas->id;
                } else {
                    echo '0';
                }
                ?>">
                <table class="form-table">
                    <tbody>
                        <tr>
                            <th scope="row">Title</th>
                            <td>
                                <input id="title" type="text" name="title" placeholder="Step name" value="<?php
                                if ($datas) {
                                    echo $datas->title;
                                }
                                ?>">
                                <label for="title"> <span class="description">This is the step name</span> </label></td>
                        </tr>
                        <tr>
                            <th scope="row">Order</th>
                            <td>
                                <input id="order" type="number" name="order" placeholder="Step order" value="<?php
                                if ($datas) {
                                    echo $datas->ordersort;
                                } else {
                                    echo '0';
                                }
                                ?>">
                                <label for="order"> <span class="description">Steps take place according to the order defined</span> </label></td>
                        </tr>
                        <tr>
                            <th scope="row">Requires at least one selection</th>
                            <td>
                                <select id="itemRequired" name="itemRequired">
                                    <option value="0" <?php
                                    if ($datas && $datas->itemRequired == 0) {
                                        echo 'selected';
                                    }
                                    ?>>No</option>
                                    <option value="1" <?php
                                    if ($datas && $datas->itemRequired == 1) {
                                        echo 'selected';
                                    }
                                    ?>>Yes</option>
                                </select><label for="itemRequired"> <span class="description">If true, the user must select at least one item to continue</span> </label></td>
                        </tr>
                        <tr>
                            <th scope="row"></th>
                            <td>
                                <input type="submit" value="Save" class="button-primary"/>
                            </td>
                        </tr>
                    </tbody>
                </table>
            </form>
        </div>
        <script>
            function qc_process(e) {

                var data = {
                    action: "step_save",
                    id: e["id"].value,
                    title: e["title"].value,
                    order: e["order"].value,
                    itemRequired: e["itemRequired"].value
                };
                jQuery("#wpefc_response").hide();
                jQuery.post(ajaxurl, data, function(response) {
                    jQuery("#wpefc_response").html(response);
                    jQuery("#wpefc_response").fadeIn();
                });

            }

        </script>
        <?php
    }

    /**
     * Menu export render
     * @return void
     */
    function estimation_submenu_export() {
        global $wpdb;

        if (!is_dir(plugin_dir_path(__FILE__) . '../tmp')) {
            mkdir(plugin_dir_path(__FILE__) . '../tmp');
            chmod(plugin_dir_path(__FILE__) . '../tmp', 0747);
        }

        $destination = plugin_dir_path(__FILE__) . '../tmp/export_estimation_form.zip';
        $zip = new ZipArchive();
        if (file_exists($destination)) {
            unlink($destination);
        }
        if ($zip->open($destination, ZipArchive::CREATE) !== true) {
            return false;
        }

        $jsonExport = array();
        $table_name = $wpdb->prefix . "wpefc_settings";
        $settings = $wpdb->get_results("SELECT * FROM $table_name WHERE id=1 LIMIT 1");
        $jsonExport['settings'] = $settings;

        $table_name = $wpdb->prefix . "wpefc_steps";
        $steps = array();
        foreach ($wpdb->get_results("SELECT * FROM $table_name") as $key => $row) {
            $steps[] = $row;
        }

        $jsonExport['steps'] = $steps;
        $table_name = $wpdb->prefix . "wpefc_items";
        $items = array();
        foreach ($wpdb->get_results("SELECT * FROM $table_name") as $key => $row) {
            $items[] = $row;
            if ($row->image != "") {
                $original_image = $row->image;
                $upload_dir = wp_upload_dir();
                $pos1 = strrpos($row->image, '/');
                $pos2 = strrpos($row->image, '/', 0 - (strlen($row->image) - $pos1) - 1);
                $pos3 = strrpos($row->image, '/', 0 - (strlen($row->image) - $pos2) - 1);
                $row->image = substr($row->image, $pos3);
                $imagePath = $upload_dir['path'] . '/../..' . substr($original_image, $pos3);
                $zip->addfile($imagePath, substr($original_image, $pos1 + 1));
            }
        }

        $jsonExport['items'] = $items;
        $fp = fopen(plugin_dir_path(__FILE__) . '../tmp/export_estimation_form.json', 'w');
        fwrite($fp, json_encode($jsonExport));
        fclose($fp);

        $zip->addfile(plugin_dir_path(__FILE__) . '../tmp/export_estimation_form.json', 'export_estimation_form.json');
        $zip->close();
        ?>
        <div class="wrap wpeExport">
            <h2>Export data</h2>
            <p>
                Export all this plugin datas to a zip file will can be imported on another website.
            </p>
            <p>
                <a download class="button-primary" href="<?php echo esc_url(trailingslashit(plugins_url('/', $this->parent->file))) . 'tmp/export_estimation_form.zip'; ?>">Export</a>
            </p>
        </div>
        <?php
    }

    /**
     * Menu import render
     * @return void
     */
    function estimation_submenu_import() {
        global $wpdb;
        ?>
        <div class="wrap wpeImport">
            <h2>Import data</h2>
            <?php
            $displayForm = true;
            $settings = $this->getSettings();
            $pageID = $settings->form_page_id;
            if (isset($_GET['import']) && isset($_FILES['importFile'])) {
                $error = false;
                if (!is_dir(plugin_dir_path(__FILE__) . '../tmp')) {
                    mkdir(plugin_dir_path(__FILE__) . '../tmp');
                    chmod(plugin_dir_path(__FILE__) . '../tmp', 0747);
                }
                $target_path = plugin_dir_path(__FILE__) . '../tmp/export_estimation_form.zip';
                if (@move_uploaded_file($_FILES['importFile']['tmp_name'], $target_path)) {


                    $upload_dir = wp_upload_dir();
                    if (!is_dir($upload_dir['path'])) {
                        mkdir($upload_dir['path']);
                    }

                    $zip = new ZipArchive;
                    $res = $zip->open($target_path);
                    if ($res === TRUE) {
                        $zip->extractTo(plugin_dir_path(__FILE__) . '../tmp/');
                        $zip->close();


                        $jsonfilename = 'export_estimation_form.json';
                        if (!file_exists(plugin_dir_path(__FILE__) . '../tmp/export_estimation_form.json')) {
                            $jsonfilename = 'export_estimation_form';
                        }

                        $file = file_get_contents(plugin_dir_path(__FILE__) . '../tmp/' . $jsonfilename);
                        $dataJson = json_decode($file, true);

                        $table_name = $wpdb->prefix . "wpefc_settings";
                        $wpdb->query("TRUNCATE TABLE $table_name");
                        foreach ($dataJson['settings'] as $key => $value) {
                            $value['form_page_id'] = $pageID;
                            if ($value['item_pictures_size'] == 0) {
                                $value['item_pictures_size'] = 64;
                            }
                            $wpdb->insert($table_name, $value);
                        }

                        $table_name = $wpdb->prefix . "wpefc_steps";
                        $wpdb->query("TRUNCATE TABLE $table_name");
                        foreach ($dataJson['steps'] as $key => $value) {
                            $wpdb->insert($table_name, $value);
                        }

                        $table_name = $wpdb->prefix . "wpefc_items";
                        $wpdb->query("TRUNCATE TABLE $table_name");
                        foreach ($dataJson['items'] as $key => $value) {

                            if ($value['image'] && $value['image'] != "") {
                                if (!file_exists($upload_dir['path'] . '/../..' . $value['image'])) {
                                    $year = substr($value['image'], 0, strpos($value['image'], '/', 1));
                                    $mouth = '/' . substr($value['image'], strlen($year) + 1, 2);
                                    if (!is_dir($upload_dir['path'] . '/../..' . $year)) {
                                        mkdir($upload_dir['path'] . '/../..' . $year);
                                    }
                                    if (!is_dir($upload_dir['path'] . '/../..' . $year . $mouth)) {
                                        mkdir($upload_dir['path'] . '/../..' . $year . $mouth);
                                    }
                                    $img_name = substr($value['image'], strrpos($value['image'], '/') + 1);
                                    copy(plugin_dir_path(__FILE__) . '../tmp/' . $img_name, $upload_dir['path'] . '/../..' . $year . $mouth . '/' . $img_name);
                                    unlink(plugin_dir_path(__FILE__) . '../tmp/' . $img_name);
                                }
                                $value['image'] = $upload_dir['baseurl'] . $value['image'];
                            }

                            $wpdb->insert($table_name, $value);
                        }
                        $files = glob(plugin_dir_path(__FILE__) . '../tmp/*');
                        foreach ($files as $file) {
                            if (is_file($file))
                                unlink($file);
                        }

                        $this->updateCSS();
                    } else {
                        $error = true;
                    }
                } else {
                    $error = true;
                }
                if ($error) {
                    echo '<div class="error">An error occurred during the transfer</div>';
                } else {
                    $displayForm = false;
                    echo '<div class="updated">Data has been imported.</div>';
                }
            }
            if ($displayForm) {
                ?>
                <p>
                    Import here the zip file created using the "Export" tool.
                </p>
                <div class="error" style="color: red;">
                    WARNING: import data will overwrite existing ones!
                </div>
                <form action="admin.php?page=wpe-import&import=1" method="post" enctype="multipart/form-data">
                    <p>
                        <input id="importFile" type="file" name="importFile" placeholder="Select the .zip file"/>
                        <label for="importFile"> <span class="description">Select the generated .zip file</span> </label>
                    </p>
                    <p>
                        <button type="submit" class="button-primary">
                            Import
                        </button>
                    </p>
                </form>
                <?php
            }
            ?>
        </div>
        <?php
    }

    /**
     * delete a folder
     * @return void
     */
    private function deleteDir($dirPath) {
        if (!is_dir($dirPath)) {
            throw new InvalidArgumentException("$dirPath must be a directory");
        }
        if (substr($dirPath, strlen($dirPath) - 1, 1) != '/') {
            $dirPath .= '/';
        }
        $files = glob($dirPath . '*', GLOB_MARK);
        foreach ($files as $file) {
            if (is_dir($file)) {
                self::deleteDir($file);
            } else {
                unlink($file);
            }
        }
        rmdir($dirPath);
    }

    /**
     * Menu settings render
     * @return void
     */
    function estimation_submenu_settings() {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_settings";
        $settings = $wpdb->get_results("SELECT * FROM $table_name WHERE id=1 LIMIT 1");
        $settings = $settings[0];

        if (isset($_GET['remove-field'])) {
            $this->remove_field($_GET['remove-field']);
        }

        $fields = $this->getFieldsDatas();
        ?>
        <div class="wrap wpeSettings">
            <h2>Flat Cost Estimation Form Settings</h2>
            <div id="wpefc_response"></div>
            <h2 class="nav-tab-wrapper">
                <?php
                if (isset($_GET['tab']))
                    $tab = $_GET['tab'];
                else
                    $tab = 'general';
                $active = '';
                if ($tab == 'general') {
                    $active = 'nav-tab-active';
                }
                echo '<a class="nav-tab ' . $active . '" href="?page=wpefc_menu&tab=general">General</a>';
                $active = '';
                if ($tab == 'intro') {
                    $active = 'nav-tab-active';
                }
                echo '<a class="nav-tab ' . $active . '" href="?page=wpefc_menu&tab=intro">Introduction</a>';
                $active = '';
                if ($tab == 'last') {
                    $active = 'nav-tab-active';
                }
                echo '<a class="nav-tab ' . $active . '" href="?page=wpefc_menu&tab=last">Last Step</a>';
                ?>
            </h2>
            <form method="post" action="#" id="form_settings" onsubmit="qc_process(this);
                            return false;">
                <table class="form-table">
                    <tbody>
                        <?php
                        switch ($tab) {
                            case 'general' :
                                ?>
                                <tr>
                                    <th colspan="2"><h3>General</h3></th>
                            </tr>
                            <tr>
                                <th scope="row">Color A</th>
                                <td>
                                    <input id="colorA" type="text" name="colorA" class="colorpick" placeholder="Enter the color hex" value="<?php echo $settings->colorA; ?>">
                                    <label for="colorA">
                                        <span class="description">ex : #1abc9c</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Color B</th>
                                <td>
                                    <input id="colorB" type="text" name="colorB" class="colorpick" placeholder="Enter the color hex" value="<?php echo $settings->colorB; ?>">
                                    <label for="colorB">
                                        <span class="description">ex : #34495e</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Initial price</th>
                                <td>
                                    <input id="initial_price" type="number" name="initial_price" placeholder="Initial price" value="<?php echo $settings->initial_price; ?>">
                                    <label for="initial_price">
                                        <span class="description">Starting price</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Currency</th>
                                <td>
                                    <input id="currency" type="text" name="currency" placeholder="Currency" value="<?php echo $settings->currency; ?>">
                                    <label for="currency">
                                        <span class="description">$, € , £ ...</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Pictures size</th>
                                <td>
                                    <input id="item_pictures_size" type="number" name="item_pictures_size" placeholder="Item pictures size" value="<?php echo $settings->item_pictures_size; ?>">
                                    <label for="item_pictures_size">
                                        <span class="description">Enter a size in pixels (ex : 64)</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Succeed text</th>
                                <td>
                                    <input id="succeed_text" type="text" name="succeed_text" placeholder="Final text" value="<?php echo $settings->succeed_text; ?>">
                                    <label for="succeed_text">
                                        <span class="description">Something like "Thanks, we will contact you soon"</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Selection required</th>
                                <td>
                                    <input id="errorMessage" type="text" name="errorMessage" placeholder="Error message" value="<?php echo $settings->errorMessage; ?>">
                                    <label for="errorMessage">
                                        <span class="description">Something like "You need to select an item to continue"</span>
                                    </label>
                                </td>
                            </tr>

                            <tr>
                                <th scope="row">Buttons "next step" label</th>
                                <td>
                                    <input id="btn_step" type="text" name="btn_step" placeholder="Button label" value="<?php echo $settings->btn_step; ?>">
                                    <label for="btn_step">
                                        <span class="description">Something like "NEXT STEP"</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Link "previous step" text</th>
                                <td>
                                    <input id="previous_step" type="text" name="previous_step" placeholder="Return link text" value="<?php echo $settings->previous_step; ?>">
                                    <label for="previous_step">
                                        <span class="description">Something like "return to previous step"</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Call an url on close</th>
                                <td>
                                    <input id="close_url" type="text" name="close_url" placeholder="http://www..." value="<?php echo $settings->close_url; ?>">
                                    <label for="close_url">
                                        <span class="description">Complete this field if you want to call a specific url on close. Else enter leave blank.</span>
                                    </label>
                                </td>
                            </tr>

                            <tr>
                                <th colspan="2"><h3>Email</h3></th>
                            </tr>
                            <tr>
                                <th scope="row">Admin email</th>
                                <td>
                                    <input id="email" type="text" name="email" placeholder="Email adress" value="<?php echo $settings->email; ?>">
                                    <label for="email">
                                        <span class="description">Email that will receive requests</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Email subject</th>
                                <td>
                                    <input id="email_subject" type="text" name="email_subject" placeholder="Email subject" value="<?php echo $settings->email_subject; ?>">
                                    <label for="email_subject">
                                        <span class="description">Something like "New order from your website"</span>
                                    </label>
                                </td>
                            </tr>
                            <?php
                            if (is_plugin_active('woocommerce/woocommerce.php')) {
                                $disp = '';
                            } else {
                                $disp = 'style="display:none;"';
                            }
                            ?>
                            <tr <?php echo $disp; ?>>
                                <th colspan="2"><h3>Woo Commerce</h3></th>
                            </tr>
                            <tr <?php echo $disp; ?>>
                                <th scope="row">Add selected items to cart</th>
                                <td>
                                    <select id="save_to_cart" name="save_to_cart">
                                        <option value="0" <?php
                                        if ($settings->save_to_cart == 0) {
                                            echo 'selected';
                                        }
                                        ?>>No</option>
                                        <option value="1" <?php
                                        if ($settings->save_to_cart == 1) {
                                            echo 'selected';
                                        }
                                        ?>>Yes</option>
                                    </select>
                                    <label for="save_to_cart">
                                        <span class="description">If true, all items must beings products of the woo catalog </span>
                                    </label>
                                </td>
                            </tr>
                            <?php
                            break;
                        case 'intro' :
                            ?>
                            <tr>
                                <th colspan="2"><h3>Introduction</h3></th>
                            </tr>
                            <tr>
                                <th scope="row">Is Introduction activated ?</th>
                                <td>
                                    <select id="intro_enabled" name="intro_enabled">
                                        <option value="0" <?php
                                        if ($settings->intro_enabled == 0) {
                                            echo 'selected';
                                        }
                                        ?>>No</option>
                                        <option value="1" <?php
                                        if ($settings->intro_enabled == 1) {
                                            echo 'selected';
                                        }
                                        ?>>Yes</option>
                                    </select>
                                    <label for="intro_enabled">
                                        <span class="description">Is Introduction enabled ?</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Introduction title</th>
                                <td>
                                    <input id="intro_title" type="text" name="intro_title" placeholder="Title" value="<?php echo $settings->intro_title; ?>">
                                    <label for="intro_title">
                                        <span class="description">Something like "HOW MUCH TO MAKE MY WEBSITE ?"</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Introduction text</th>
                                <td>
                                    <input id="intro_text" type="text" name="intro_text" placeholder="Text" value="<?php echo $settings->intro_text; ?>">
                                    <label for="intro_text">
                                        <span class="description">Something like "Estimate the cost of a website easily using this awesome tool."</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Introduction button</th>
                                <td>
                                    <input id="intro_btn" type="text" name="intro_btn" placeholder="Button label" value="<?php echo $settings->intro_btn; ?>">
                                    <label for="intro_btn">
                                        <span class="description">Something like "GET STARTED"</span>
                                    </label>
                                </td>
                            </tr>
                            <?php
                            break;
                        case 'last' :
                            ?>
                            <tr>
                                <th colspan="2"><h3>Last Step</h3></th>
                            </tr>
                            <tr>
                                <th scope="row">Last step title</th>
                                <td>
                                    <input id="last_title" type="text" name="last_title" placeholder="Title" value="<?php echo $settings->last_title; ?>">
                                    <label for="last_title">
                                        <span class="description">Something like "Final cost", "Result" ...</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Last step text</th>
                                <td>
                                    <input id="last_text" type="text" name="last_text" placeholder="Text" value="<?php echo $settings->last_text; ?>">
                                    <label for="last_text">
                                        <span class="description">Something like "The final estimated price is :"</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th scope="row">Last step button</th>
                                <td>
                                    <input id="last_btn" type="text" name="last_btn" placeholder="Button label" value="<?php echo $settings->last_btn; ?>">
                                    <label for="last_btn">
                                        <span class="description">Something like "ORDER MY WEBSITE"</span>
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <th colspan="2"><h3>Final fields</h3></th>
                            </tr>
                            <?php
                            if (is_plugin_active('gravityforms/gravityforms.php')) {
                                ?>
                                <tr>
                                    <th scope="row">Use Gravityforms</th>
                                    <td>
                                        <select id="gravityFormID" name="gravityFormID">
                                            <option value="0" <?php
                                            if ($settings->gravityFormID == 0) {
                                                echo 'selected';
                                            }
                                            ?>>None</option>
                                                    <?php
                                                    $forms = RGFormsModel::get_forms(null, "title");
                                                    foreach ($forms as $form) {
                                                        $sel = '';
                                                        if ($form->id == $settings->gravityFormID) {
                                                            $sel = 'selected="selected"';
                                                        }
                                                        echo '<option ' . $sel . ' value="' . $form->id . '">' . $form->title . '</option>';
                                                    }
                                                    ?>
                                        </select>
                                        <label for="gravityFormID">
                                            <span class="description">Select a Gravity form if you want</span>
                                        </label>
                                    </td>
                                </tr>
                                <?php
                            }
                            ?>
                            <tr class="customFieldsRow">                                
                                <th scope="row">Add custom fields</th>
                                <td>
                                    <table class="wp-list-table widefat  media">
                                        <thead>
                                            <tr>
                                                <th class="manage-column column-title sortable " style="width: auto;"><a href="#">Label</a></th>
                                                <th class="manage-column column-title sortable "><a href="#">Order</a></th>
                                                <th class="manage-column column-title sortable "><a href="#">Type</a></th>
                                                <th class="manage-column column-title sortable "></th>
                                            </tr>
                                        </thead>
                                        <tbody>
                                            <?php
                                            foreach ($fields as $field) {
                                                echo '<tr>';
                                                echo '<td> <a href="admin.php?page=wpe-field-edit&field=' . $field->id . '">' . $field->label . '</a></td>';
                                                echo '<td>' . $field->ordersort . '</td>';
                                                echo '<td>' . $field->typefield . '</td>';
                                                echo '<td>';
                                                if ($field->id > 1) {
                                                    echo '<a href="admin.php?page=wpefc_menu&tab=last&remove-field=' . $field->id . '">Delete</a></td>';
                                                }
                                                echo '<td>';
                                                echo '</tr>';
                                            }
                                            ?>
                                            <tr>
                                                <td colspan="4" style="text-align: center;">
                                                    <a class="button-primary" href="admin.php?page=wpe-field-edit">Add field</button>
                                                </td>
                                            </tr>
                                        </tbody>

                                    </table>
                                </td>
                            </tr>
                            <?php
                            break;
                        default :
                    }
                    ?>      
                    <tr>
                        <th scope="row"></th>
                        <td>
                            <input type="submit" value="Save" class="button-primary"/>
                        </td>
                    </tr>
                    </tbody>
                </table>
            </form>
        </div>
        <script>
            function checkGravity() {
                if (jQuery('#gravityFormID').length > 0 && jQuery('#gravityFormID').val() > 0) {
                    jQuery('.customFieldsRow').hide();
                } else {
                    jQuery('.customFieldsRow').show();
                }
            }
            function qc_process(e) {
                var data = {action: "settings_save"};
                jQuery('#form_settings input, #form_settings select').each(function() {
                    if (jQuery(this).attr('name')) {
                        eval('data.' + jQuery(this).attr('name') + ' = jQuery(this).val();');
                    }
                });
                jQuery.post(ajaxurl, data, function(response) {
                    jQuery("#wpefc_response").html(response);
                });
            }
            jQuery(document).ready(function() {
                if (jQuery('#gravityFormID').length > 0) {
                    jQuery('#gravityFormID').change(checkGravity);
                }
                checkGravity();
            });
        </script>
        <?php
    }

    /**
     * save settings
     * @return void
     */
    function settings_save() {
        global $wpdb;
        $response = "Error, try again later.";
        $table_name = $wpdb->prefix . "wpefc_settings";

        $sqlDatas = array();
        foreach ($_POST as $key => $value) {
            if ($key != 'action') {
                $sqlDatas[$key] = stripslashes($value);
            }
        }
        $wpdb->update($table_name, $sqlDatas, array('id' => 1));
        $response = '<div id="message" class="updated"><p>Settings <strong>saved</strong>.</p></div>';

        echo $response;
        $this->updateCSS();
        die();
    }

    /**
     * save settings
     * @return void
     */
    private function updateCSS() {
        $settings = $this->getSettings();
        $colorsStyles = '
        body {
        color: ' . $settings->colorB . ';
        }
        .tooltip-inner,#mainPanel .genSlide .genContent div.selectable span.icon_quantity {
        background-color: ' . $settings->colorB . ';
        }
        .tooltip.bottom .tooltip-arrow{
        border-bottom-color: ' . $settings->colorB . ';
        }
        .btn-primary,.btn-primary:hover,.btn-primary:active, .genPrice .progress .progress-bar-price, .progress-bar,.quantityBtns a{
        background-color: ' . $settings->colorA . ';
        }
        .has-switch > div.switch-on label,.form-group.focus .form-control,.form-control:focus {
        border-color: ' . $settings->colorA . ';
        }
        a,a:hover,a:active,#mainPanel .genSlide .genContent div.selectable.checked span.icon_select,#mainPanel #finalPrice, .ginput_product_price {
        color: ' . $settings->colorA . ';
        }
        #mainPanel .genSlide .genContent div.selectable .img {
                max-width: ' . $settings->item_pictures_size . 'px;
                max-height: ' . $settings->item_pictures_size . 'px;	
        }
        ';
        $fp = fopen(plugin_dir_path(__FILE__) . '../assets/css/colors.css', 'w');
        fwrite($fp, $colorsStyles);
        fclose($fp);
    }

    /**
     * save field
     * @return void
     */
    function field_save() {
        global $wpdb;
        $response = "Error, try again later.";
        $table_name = $wpdb->prefix . "wpefc_fields";
        $id = $_POST['id'];

        $sqlDatas = array();
        foreach ($_POST as $key => $value) {
            if ($key != 'action') {
                $sqlDatas[$key] = stripslashes($value);
            }
        }

        if ($id == '0') {
            $rows_affected = $wpdb->insert($table_name, $sqlDatas);
            if ($rows_affected == 1) {
                $response = '<div id="message" class="updated"><p>Field <strong>saved</strong>.</p></div>';
            }
        } else {
            $wpdb->update($table_name, $sqlDatas, array('id' => $id));
            $response = '<div id="message" class="updated"><p>Field <strong>saved</strong>.</p></div>';
        }
        echo $response;
        die();
    }

    /**
     * save step
     * @return void
     */
    function step_save() {
        global $wpdb;
        $response = "Error, try again later.";
        $table_name = $wpdb->prefix . "wpefc_steps";
        $id = $_POST['id'];
        $title = stripslashes($_POST['title']);
        $order = $_POST['order'];
        $itemRequired = $_POST['itemRequired'];

        if ($id == '0') {
            $rows_affected = $wpdb->insert($table_name, array('title' => $title, 'ordersort' => $order, 'itemRequired' => $itemRequired));
            if ($rows_affected == 1) {
                $response = '<div id="message" class="updated"><p>Step <strong>saved</strong>.</p></div>';
            }
        } else {
            $wpdb->update($table_name, array('title' => $title, 'ordersort' => $order, 'itemRequired' => $itemRequired), array('id' => $id));
            $response = '<div id="message" class="updated"><p>Step <strong>saved</strong>.</p></div>';
        }
        echo $response;
        die();
    }

    /**
     * save item
     * @return void
     */
    function item_save() {
        global $wpdb;
        $response = "Error, try again later.";
        $table_name = $wpdb->prefix . "wpefc_items";
        $id = $_POST['id'];
        $title = stripslashes($_POST['title']);
        $order = $_POST['order'];
        $price = $_POST['price'];
        $group = stripslashes($_POST['group']);
        $image = "";
        $image = $_POST['image'];
        $stepID = $_POST['stepID'];
        $type = $_POST['type'];
        $ischecked = $_POST['ischecked'];
        $quantity_enabled = $_POST['quantity_enabled'];
        $quantity_max = $_POST['quantity_max'];
        $wooProductID = $_POST['wooProductID'];
        $imageTint = $_POST['imageTint'];
        $isRequired = $_POST['isRequired'];

        if ($id == '0') {
            $rows_affected = $wpdb->insert($table_name, array('title' => $title, 'ordersort' => $order, 'price' => $price, 'groupitems' => $group, 'image' => $image, 'stepID' => $stepID, 'type' => $type, 'ischecked' => $ischecked, 'quantity_enabled' => $quantity_enabled, 'quantity_max' => $quantity_max, 'wooProductID' => $wooProductID, 'imageTint' => $imageTint, 'isRequired' => $isRequired));
            if ($rows_affected == 1) {
                $response = '<div id="message" class="updated"><p>Item <strong>saved</strong>.</p></div>';
            }
        } else {
            $wpdb->update($table_name, array('title' => $title, 'ordersort' => $order, 'price' => $price, 'groupitems' => $group, 'image' => $image, 'stepID' => $stepID, 'type' => $type, 'ischecked' => $ischecked, 'quantity_enabled' => $quantity_enabled, 'quantity_max' => $quantity_max, 'wooProductID' => $wooProductID, 'imageTint' => $imageTint, 'isRequired' => $isRequired), array('id' => $id));
            $response = '<div id="message" class="updated"><p>Item <strong>saved</strong>.</p></div>';
        }
        echo $response;
        die();
    }

    /**
     * Main Instance
     *
     *
     * @since 1.0.0
     * @static
     * @return Main instance
     */
    public static function instance($parent) {
        if (is_null(self::$_instance)) {
            self::$_instance = new self($parent);
        }
        return self::$_instance;
    }

    // End instance()

    /**
     * Cloning is forbidden.
     *
     * @since 1.0.0
     */
    public function __clone() {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?'), $this->parent->_version);
    }

// End __clone()

    /**
     * Unserializing instances of this class is forbidden.
     *
     * @since 1.0.0
     */
    public function __wakeup() {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?'), $this->parent->_version);
    }

// End __wakeup()
}
