<?php

if (!defined('ABSPATH'))
    exit;

class WPE_Tools {

    /**
     * The single instance
     * @var 	object
     * @access  private
     * @since 	1.0.0
     */
    private static $_instance = null;

    /**
     * Settings class object
     * @var     object
     * @access  public
     * @since   1.0.0
     */
    public $settings = null;

    /**
     * The version number.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $_version;

    /**
     * The token.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $_token;

    /**
     * The main plugin file.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $file;

    /**
     * The main plugin directory.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $dir;

    /**
     * The plugin assets directory.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $assets_dir;

    /**
     * The plugin assets URL.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $assets_url;

    /**
     * Suffix for Javascripts.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $templates_url;

    /**
     * Suffix for Javascripts.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $script_suffix;

    /**
     * For menu instance
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $menu;

    /**
     * For template
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $plugin_slug;

    /**
     * Constructor function.
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    public function __construct($file = '', $version = '1.0.0') {
        $this->_version = $version;
        $this->_token = 'wpe';

        $this->file = $file;
        $this->dir = dirname($this->file);
        $this->assets_dir = trailingslashit($this->dir) . 'assets';
        $this->assets_url = esc_url(trailingslashit(plugins_url('/assets/', $this->file)));
        $this->templates_url = esc_url(trailingslashit(plugins_url('/templates/', $this->file)));

        $this->templates = array('estimation-popup-page.php' => __('Estimation Form Page', $this->plugin_slug));

        // Load admin JS & CSS
        add_action('wp_ajax_nopriv_cart_save', array($this, 'cart_save'));
        add_action('wp_ajax_cart_save', array($this, 'cart_save'));
        add_action('admin_enqueue_scripts', array($this, 'admin_enqueue_scripts'), 10, 1);
        add_action('admin_enqueue_scripts', array($this, 'admin_enqueue_styles'), 10, 1);
        add_action('wp_enqueue_scripts', array($this, 'frontend_enqueue_scripts'), 10, 1);
        add_action('wp_enqueue_scripts', array($this, 'frontend_enqueue_styles'), 10, 1);
        add_filter('page_attributes_dropdown_pages_args', array($this, 'register_project_templates'));
        add_filter('wp_insert_post_data', array($this, 'register_project_templates'));
        add_filter('template_include', array($this, 'view_project_template'));

        if (isset($_GET['page']) && $_GET['page'] == 'estimation-popup') {
            add_filter('template_include', array($this, 'load_popup_template'));
        }
        $templates = wp_get_theme()->get_page_templates();
        $templates = array_merge($templates, $this->templates);
    }

    /**
     * Adds the template to the pages cache
     *
     * @param   array    $atts    The attributes for the page attributes dropdown
     * @return  array    $atts    The attributes for the page attributes dropdown
     * @verison	1.0.0
     * @since	1.0.0
     */
    public function register_project_templates($atts) {

        $cache_key = 'page_templates-' . md5(get_theme_root() . '/' . get_stylesheet());
        $templates = wp_cache_get($cache_key, 'themes');
        if (empty($templates)) {
            $templates = array();
        }
        wp_cache_delete($cache_key, 'themes');
        $templates = array_merge($templates, $this->templates);
        wp_cache_add($cache_key, $templates, 'themes', 1800);
        return $atts;
    }

    /**
     * Load popup template.
     * @access  public
     * @since   1.0.0
     * @return void
     */
    public function load_popup_template($template) {
        $file = plugin_dir_path(__FILE__) . '../templates/estimation-popup-page.php';
        if (file_exists($file)) {
            return $file;
        }
    }

    /**
     * Checks if the template is assigned to the page
     *
     * @version	1.0.0
     * @since	1.0.0
     */
    public function view_project_template($template) {

        global $post;

        if (!isset($this->templates[get_post_meta($post->ID, '_wp_page_template', true)])) {
            return $template;
        }

        $file = plugin_dir_path(__FILE__) . '../templates/' . get_post_meta($post->ID, '_wp_page_template', true);
        if (file_exists($file)) {
            return $file;
        }
        return $template;
    }

    /**
     * Load frontend CSS.
     * @access  public
     * @since   1.0.0
     * @return void
     */
    public function frontend_enqueue_styles($hook = '') {
        wp_register_style($this->_token . '-frontend', esc_url($this->assets_url) . 'css/frontend.css', array(), $this->_version);
        wp_enqueue_style($this->_token . '-frontend');
    }

    /**
     * Load estimation form CSS.
     * @access  public
     * @since   1.0.0
     * @return void
     */
    public function estimation_form_enqueue_styles($hook = '') {
        wp_register_style($this->_token . '-reset', esc_url($this->assets_url) . 'css/reset.css', array(), $this->_version);
        wp_enqueue_style($this->_token . '-reset');
        wp_register_style($this->_token . '-bootstrap', esc_url($this->assets_url) . 'bootstrap/css/bootstrap.css', array(), $this->_version);
        wp_enqueue_style($this->_token . '-bootstrap');
        wp_register_style($this->_token . '-flat-ui', esc_url($this->assets_url) . 'css/flat-ui.css', array(), $this->_version);
        wp_enqueue_style($this->_token . '-flat-ui');
        wp_register_style($this->_token . '-estimationpopup', esc_url($this->assets_url) . 'css/estimation_popup.css', array(), $this->_version);
        wp_enqueue_style($this->_token . '-estimationpopup');
        wp_register_style($this->_token . '-colors', esc_url($this->assets_url) . 'css/colors.css', array(), $this->_version);
        wp_enqueue_style($this->_token . '-colors');
    }

    /**
     * Load frontend Javascript.
     * @access  public
     * @since   1.0.0
     * @return void
     */
    public function frontend_enqueue_scripts($hook = '') {
        wp_register_script($this->_token . '-frontend', esc_url($this->assets_url) . 'js/frontend.js', array('jquery'), $this->_version);
        wp_enqueue_script($this->_token . '-frontend');
        $settings = $this->getSettings();
        wp_localize_script($this->_token . '-frontend', 'form_page_id', $settings->form_page_id);
        wp_localize_script($this->_token . '-frontend', 'close_url', $settings->close_url);
    }

    /**
     * Load estimation form Javascript.
     * @access  public
     * @since   1.0.0
     * @return void
     */
    public function estimation_form_enqueue_scripts($hook = '') {
        $settings = $this->getSettings();
        wp_register_script($this->_token . '-jquery-ui', esc_url($this->assets_url) . 'js/jquery-ui-1.10.3.custom.min.js', array('jquery'), $this->_version);
        wp_enqueue_script($this->_token . '-jquery-ui');
        wp_register_script($this->_token . '-jquery-touch-punch', esc_url($this->assets_url) . 'js/jquery.ui.touch-punch.min.js', array('jquery'), $this->_version);
        wp_enqueue_script($this->_token . '-jquery-touch-punch');
        wp_register_script($this->_token . '-bootstrap', esc_url($this->assets_url) . 'js/bootstrap.min.js', array('jquery'), $this->_version);
        wp_enqueue_script($this->_token . '-bootstrap');
        wp_register_script($this->_token . '-bootstrap-select', esc_url($this->assets_url) . 'js/bootstrap-select.js', array(), $this->_version);
        wp_enqueue_script($this->_token . '-bootstrap-select');
        wp_register_script($this->_token . '-bootstrap-switch', esc_url($this->assets_url) . 'js/bootstrap-switch.js', array('jquery'), $this->_version);
        wp_enqueue_script($this->_token . '-bootstrap-switch');
        wp_register_script($this->_token . '-flatui-checkbox', esc_url($this->assets_url) . 'js/flatui-checkbox.js', array(), $this->_version);
        wp_enqueue_script($this->_token . '-flatui-checkbox');
        wp_register_script($this->_token . '-flatui-radio', esc_url($this->assets_url) . 'js/flatui-radio.js', array(), $this->_version);
        wp_enqueue_script($this->_token . '-flatui-radio');
        wp_register_script($this->_token . '-jquery-tagsinput', esc_url($this->assets_url) . 'js/jquery.tagsinput.js', array(), $this->_version);
        wp_enqueue_script($this->_token . '-jquery-tagsinput');
        wp_register_script($this->_token . '-jquery-placeholder', esc_url($this->assets_url) . 'js/jquery.placeholder.js', array(), $this->_version);
        wp_enqueue_script($this->_token . '-jquery-placeholder');
        wp_register_script($this->_token . '-flatui-app', esc_url($this->assets_url) . 'js/application.js', array(), $this->_version);
        wp_enqueue_script($this->_token . '-flatui-app');


        if ($settings->gravityFormID > 0) {
            wp_register_script($this->_token . '-gforms_ui_datepicker', esc_url($this->assets_url) . '../gravityforms/js/jquery-ui/ui.datepicker.js', array(), $this->_version);
            wp_enqueue_script('-gforms_ui_dateicker');
            wp_register_script('gforms_datepicker', esc_url($this->assets_url) . '../gravityforms/js/datepicker.js', array("gforms_ui_datepicker"), $this->_version);
            wp_enqueue_script('gforms_datepicker');
            wp_register_script('gforms_conditional_logic_lib', esc_url($this->assets_url) . '../gravityforms/js/conditional_logic.js', array("gforms_ui_datepicker"), $this->_version);
            wp_enqueue_script('gforms_conditional_logic_lib');
            wp_enqueue_script('gform_gravityforms');
        }
        //wp_enqueue_style("gforms_css", WP_PLUGIN_URL . "/gravityforms/css/forms.css");

        wp_register_script($this->_token . '-estimationpopup', esc_url($this->assets_url) . 'js/estimation_popup.js', array('jquery'), $this->_version);
        wp_enqueue_script($this->_token . '-estimationpopup');

        $settings = $this->getSettings();

        wp_localize_script($this->_token . '-estimationpopup', 'ajaxurl', admin_url('admin-ajax.php'));
        wp_localize_script($this->_token . '-estimationpopup', 'initialPrice', $settings->initial_price);
        wp_localize_script($this->_token . '-estimationpopup', 'currency', $settings->currency);
        wp_localize_script($this->_token . '-estimationpopup', 'emailUrl', esc_url($this->assets_url) . 'formReception.php');
        wp_localize_script($this->_token . '-estimationpopup', 'admin_email', $settings->email);
        wp_localize_script($this->_token . '-estimationpopup', 'email_subject', $settings->email_subject);
        wp_localize_script($this->_token . '-estimationpopup', 'intro_enabled', $settings->intro_enabled);
        wp_localize_script($this->_token . '-estimationpopup', 'save_to_cart', $settings->save_to_cart);
        wp_localize_script($this->_token . '-estimationpopup', 'colorA', $settings->colorA);
        wp_localize_script($this->_token . '-estimationpopup', 'colorB', $settings->colorB);
        wp_localize_script($this->_token . '-estimationpopup', 'close_url', $settings->close_url);
    }

    /**
     * Load admin CSS.
     * @access  public
     * @since   1.0.0
     * @return void
     */
    public function admin_enqueue_styles($hook = '') {
        wp_register_style($this->_token . '-colpick', esc_url($this->assets_url) . 'css/colpick.css', array(), $this->_version);
        wp_enqueue_style($this->_token . '-colpick');
        wp_register_style($this->_token . '-admin', esc_url($this->assets_url) . 'css/admin.css', array(), $this->_version);
        wp_enqueue_style($this->_token . '-admin');
    }

// End admin_enqueue_styles()

    /**
     * Load admin Javascript.
     * @access  public
     * @since   1.0.0
     * @return void
     */
    public function admin_enqueue_scripts($hook = '') {
        wp_register_script($this->_token . '-colpick', esc_url($this->assets_url) . 'js/colpick.js', array('jquery'), $this->_version);
        wp_enqueue_script($this->_token . '-colpick');
        wp_register_script($this->_token . '-admin', esc_url($this->assets_url) . 'js/admin.js', array('jquery'), $this->_version);
        wp_enqueue_script($this->_token . '-admin');
    }

// End admin_enqueue_scripts()

    /**
     * Main WPE_Tools Instance
     *
     *
     * @since 1.0.0
     * @static
     * @see WPE_Tools()
     * @return Main WPE_Tools instance
     */
    public static function instance($file = '', $version = '1.0.0') {
        if (is_null(self::$_instance)) {
            self::$_instance = new self($file, $version);
        }
        return self::$_instance;
    }

// End instance()

    /**
     * Cloning is forbidden.
     *
     * @since 1.0.0
     */
    public function __clone() {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?'), $this->_version);
    }

// End __clone()

    /**
     * Unserializing instances of this class is forbidden.
     *
     * @since 1.0.0
     */
    public function __wakeup() {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?'), $this->_version);
    }

// End __wakeup()

    /**
     * Return settings.
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    public function getSettings() {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_settings";
        $settings = $wpdb->get_results("SELECT * FROM $table_name WHERE id=1 LIMIT 1");
        return $settings[0];
    }

// End getSettings()

    /**
     * Return steps data.
     * @access  public
     * @since   1.0.0
     * @return  object
     */
    public function getStepsData() {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_steps";
        $rows = $wpdb->get_results("SELECT * FROM $table_name ORDER BY ordersort");
        return $rows;
    }

// End getStepsData()

    /**
     * Return items data.
     * @access  public
     * @since   1.0.0
     * @return  object
     */
    public function getItemsData() {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_items";
        $rows = $wpdb->get_results("SELECT * FROM $table_name ORDER BY ordersort");
        return $rows;
    }

// End getItemsData()

    /**
     * Get  fields datas
     * @since   1.6.0
     * @return object
     */
    public function getFieldsData() {
        global $wpdb;
        $table_name = $wpdb->prefix . "wpefc_fields";
        $rows = $wpdb->get_results("SELECT * FROM $table_name  ORDER BY ordersort ASC");
        return $rows;
    }

    /**
     * Save form datas to cart (woocommerce only)
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    public function cart_save() {
        global $woocommerce;
        $products = $_POST['products'];
        foreach ($products as $product) {
            $productWoo = new WC_Product($product['product_id']);
            $existInCart = false;
            foreach ($woocommerce->cart->get_cart() as $cart_item_key => $cart_item) {
                if ($cart_item['product_id'] == $product['product_id']) {
                    $cart_item['quantity'] += $product['quantity'];
                }
            }
            if (!$existInCart) {
                $woocommerce->cart->add_to_cart($product['product_id'], $product['quantity']);
            }
        }

        // print_r($products);
    }


    /**
     * Log the plugin version number.
     * @access  public
     * @since   1.0.0
     * @return  void
     */
    private function _log_version_number() {
        update_option($this->_token . '_version', $this->_version);
    }

}
